import {MariadbService} from "@services/databases/mariadb.service";
//import {MongodbService} from "@services/databases/mongodb.service";
import {LogsUtils} from "@utils/logs.util";
import {UserInDatabase} from "@interfaces/db/mariadb.interface";

interface MariaDbStatusResult {
	fieldCount: number;
	affectedRows: number;
	insertId: number;
	info: string;
	serverStatus: number;
	warningStatus: number;
	changedRows: number;
}

export class DatabasesService {
	private readonly _appName;
	private readonly maria: MariadbService;
	//private readonly mongo: MongodbService;
	private readonly logs: LogsUtils;
	constructor(appName?: string) {
		this._appName = appName || 'App';
		this.maria = new MariadbService(this._appName + ' DbInteractions');
		//this.mongo = new MongodbService(this._appName + ' DbInteractions');
		this.logs = new LogsUtils(this._appName + ' DbInteractions');
	}

	async getAllUsers() {
		const result: Array<object> = await this.maria.query("SELECT * FROM users") as unknown as Array<object>;
		this.logs.debug('Fetching users from database...', `${result?.length} user(s) found.`)
    return result;
	}

	async getUserById(id: string) {
      const result: Array<object> = await this.maria.execute(`SELECT * FROM users WHERE id = ?`, [id]) as unknown as Array<object>;
      this.logs.debug(`Fetching user with id ${id} from database...`, `${result?.length} user(s) found.`);
      return result;
  }

	async getUserByUsername(username: string) {
		const result: Array<object> = await this.maria.execute(`SELECT * FROM users WHERE username = ?`, [username]) as unknown as Array<object>;
    this.logs.debug(`Fetching user with username "${username}" from database...`, `${result?.length} user(s) found.`);
    return result;
	}

	async getUserByEmail(email: string) {
		const result: Array<object> = await this.maria.execute(`SELECT * FROM users WHERE email = ?`, [email]) as unknown as Array<object>;
    this.logs.debug(`Fetching user with email "${email}" from database...`, `${result?.length} user(s) found.`);
    return result;
	}

	async insertUser(user: UserInDatabase): Promise<boolean> {
		const factorized = await this.maria.factorize({
			values: user,
			actionName: 'Inserting a user',
			throwOnError: true
		});
		//TODO if no id stop
		const valuesArray = factorized._valuesArray;
		const questionMarks = factorized._questionMarksFields
		const keysArray = factorized._keysTemplate;
		const _sql = `INSERT INTO users (${keysArray}) VALUES (${questionMarks})`
		try {
			const result = await this.maria.execute(_sql, valuesArray) as unknown as MariaDbStatusResult
			this.logs.debug(`Inserted new user with id ${user.id}`, `Rows affected: ${result.affectedRows}`);
			return true
		} catch (err) {
			this.logs.softError('An error occurred.', err)
			return false
		}
  }
	//ToTest
	async editUser(userId: string, data: object): Promise<object> {
		const factorized = await this.maria.factorize({
      values: data,
      actionName: 'Editing a user',
      throwOnError: true
    });
    const valuesArray = factorized._valuesArray;
    const keysArray = factorized._keysTemplate.split(',');
    const setFields = keysArray.map((key) => `${key} = ?`).join(', ');
    const _sql = `UPDATE users SET ${setFields} WHERE id = ?`;
    valuesArray.push(userId);
    try {
      const result = await this.maria.execute(_sql, valuesArray) as unknown as MariaDbStatusResult;
      this.logs.debug(`Edited user with id ${userId}`, `Rows affected: ${result.affectedRows}`);
      return result;
    } catch (err) {
      this.logs.softError('An error occurred.', err);
      return {};
    }
	}
}

export const justForTesting = new DatabasesService('OnlyDevs')